@@ -32,9 +32,11 @@ if not os.path.exists(_config_path):
 
 if not os.path.exists(_secrets_path):
     _secrets = {
-        "quandl_api_key":None,
-        "tinkoff_token":None,
-        "tinkoff_broker_account_id":None
+        "quandl_api_key": os.getenv("QUANDL_API_KEY") or None,
+        "tinkoff_token": os.getenv("TINKOFF_TOKEN") or None,
+        "tinkoff_broker_account_id": os.getenv("TINKOFF_BROKER_ACCOUNT_ID") or None,
+        "mongodb_adminusername": os.getenv("MONGODB_ADMINUSERNAME") or None,
+        "mongodb_adminpassword": os.getenv("MONGODB_ADMINPASSWORD") or None,
     }
 
     try:
@@ -53,4 +55,10 @@ if not os.path.exists(_tickers_path):
         pass
 
 
-   
+_models_dir = os.path.join(_ml_investments_dir, 'models')
+if not os.path.exists(_models_dir):
+    try:
+        os.makedirs(_models_dir)
+    except OSError:
+        pass
+ 
