@@ -2,11 +2,10 @@ import argparse
 import os
 import lightgbm as lgbm
 import catboost as ctb
+
+from typing import Optional
 from urllib.request import urlretrieve
 from ml_investment.utils import load_config
-from ml_investment.data_loaders.sf1 import SF1BaseData, SF1DailyData, \
-                                           SF1QuarterlyData
-from ml_investment.data_loaders.quandl_commodities import QuandlCommoditiesData
 from ml_investment.features import QuarterlyFeatures, BaseCompanyFeatures, \
                                    FeatureMerger, DailyAggQuarterFeatures
 from ml_investment.targets import QuarterlyTarget
@@ -18,11 +17,12 @@ from ml_investment.download_scripts import download_sf1, download_commodities
 config = load_config()
 
 
-URL = 'https://github.com/fartuk/ml_investment/releases\
-      /download/weights/fair_marketcap_sf1.pickle'
+URL = 'https://github.com/fartuk/ml_investment/releases/download/weights/fair_marketcap_sf1.pickle'
 OUT_NAME = 'fair_marketcap_sf1'
+DATA_SOURCE='sf1'
 CURRENCY = 'USD'
 MAX_BACK_QUARTER = 20
+MIN_BACK_QUARTER = 0
 BAGGING_FRACTION = 0.7
 MODEL_CNT = 20
 FOLD_CNT = 5
@@ -66,11 +66,18 @@ def _check_download_data():
 
 
 def _create_data():
+    if DATA_SOURCE == 'sf1':
+        from ml_investment.data_loaders.quandl_commodities import QuandlCommoditiesData
+        from ml_investment.data_loaders.sf1 import SF1BaseData, SF1DailyData, \
+                                                   SF1QuarterlyData
+    elif DATA_SOURCE == 'mongo':
+        from ml_investment.data_loaders.mongo import SF1BaseData, SF1DailyData, \
+                                    SF1QuarterlyData, QuandlCommoditiesData        
     data = {}
-    data['quarterly'] = SF1QuarterlyData(config['sf1_data_path'])
-    data['base'] = SF1BaseData(config['sf1_data_path'])
-    data['daily'] = SF1DailyData(config['sf1_data_path'])
-    data['commodities'] = QuandlCommoditiesData(config['commodities_data_path'])
+    data['quarterly'] = SF1QuarterlyData()
+    data['base'] = SF1BaseData()
+    data['daily'] = SF1DailyData()
+    data['commodities'] = QuandlCommoditiesData()
     
     return data
 
@@ -79,7 +86,8 @@ def _create_feature():
     fc1 = QuarterlyFeatures(data_key='quarterly',
                             columns=QUARTER_COLUMNS,
                             quarter_counts=QUARTER_COUNTS,
-                            max_back_quarter=MAX_BACK_QUARTER)
+                            max_back_quarter=MAX_BACK_QUARTER,
+                            min_back_quarter=MIN_BACK_QUARTER)
 
     fc2 = BaseCompanyFeatures(data_key='base', cat_columns=CAT_COLUMNS)
 
@@ -89,13 +97,15 @@ def _create_feature():
                                   quarterly_data_key='quarterly',
                                   columns=DAILY_AGG_COLUMNS,
                                   agg_day_counts=AGG_DAY_COUNTS,
-                                  max_back_quarter=MAX_BACK_QUARTER)
+                                  max_back_quarter=MAX_BACK_QUARTER,
+                                  min_back_quarter=MIN_BACK_QUARTER)
 
     fc4 = DailyAggQuarterFeatures(daily_data_key='commodities',
                                   quarterly_data_key='quarterly',
                                   columns=['price'],
                                   agg_day_counts=AGG_DAY_COUNTS,
                                   max_back_quarter=MAX_BACK_QUARTER,
+                                  min_back_quarter=MIN_BACK_QUARTER,
                                   daily_index=COMMODITIES_CODES)
     
     feature = FeatureMerger(fc1, fc2, on='ticker')
@@ -128,7 +138,10 @@ def _create_model():
 
 
 
-def FairMarketcapSF1(pretrained=True) -> Pipeline:
+def FairMarketcapSF1(max_back_quarter: int=None,
+                     min_back_quarter: int=None,
+                     data_source: Optional[str]=None,
+                     pretrained: bool=True) -> Pipeline:
     '''
     Model is used to estimate fair company marketcap for several last quarters. 
     Pipeline uses features from 
@@ -151,12 +164,36 @@ def FairMarketcapSF1(pretrained=True) -> Pipeline:
 
     Parameters
     ----------
+    max_back_quarter:
+        max quarter number which will be used in model
+    min_back_quarter:
+        min quarter number which will be used in model
+    data_source:
+        which data use for model. One of ['sf1', 'mongo'].
+        If 'mongo', than data will be loaded from db,
+        credentials specified at `~/.ml_investment/config.json`.
+        If 'sf1' - from folder specified at ``sf1_data_path``
+        in `~/.ml_investment/secrets.json`.
     pretrained:
-        use pretreined weights or not. If so, `fair_marketcap_sf1.pickle`
-        will be downloaded. Downloading directory path can be changed in
+        use pretreined weights or not.  
+        Downloading directory path can be changed in
         `~/.ml_investment/config.json` ``models_path``
     '''
-    _check_download_data()
+    if data_source is not None:
+        global DATA_SOURCE 
+        DATA_SOURCE = data_source
+        
+    if max_back_quarter is not None:
+        global MAX_BACK_QUARTER 
+        MAX_BACK_QUARTER = max_back_quarter
+
+    if min_back_quarter is not None:
+        global MIN_BACK_QUARTER 
+        MIN_BACK_QUARTER = min_back_quarter
+
+    if DATA_SOURCE == 'sf1':
+        _check_download_data()
+        
     data = _create_data()
     feature = _create_feature()
     target = _create_target()
@@ -178,17 +215,13 @@ def FairMarketcapSF1(pretrained=True) -> Pipeline:
     return pipeline
 
 
-
-
-
-
-def main():
+def main(data_source):
     '''
     Default model training. Resulted model weights directory path 
     can be changed in `~/.ml_investment/config.json` ``models_path``
     '''
-    pipeline = FairMarketcapSF1(pretrained=False)
-    base_df = SF1BaseData(config['sf1_data_path']).load()
+    pipeline = FairMarketcapSF1(pretrained=False, data_source=data_source)    
+    base_df = pipeline.data['base'].load()
     tickers = base_df[(base_df['currency'] == CURRENCY) &\
                       (base_df['scalemarketcap'].apply(lambda x: x in SCALE_MARKETCAP))
                      ]['ticker'].values
@@ -199,5 +232,9 @@ def main():
 
 
 if __name__ == '__main__':
-   main() 
+    parser = argparse.ArgumentParser()
+    arg = parser.add_argument
+    arg('--data_source', type=str)
+    args = parser.parse_args()
+    main(args.data_source)
     
